/*
 * SPDX-FileCopyrightText: Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: BSD-3-Clause
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef __FLEXIO_OS_H__
#define __FLEXIO_OS_H__

#include <stdint.h>

typedef enum flexio_os_errno {
    FLEXIO_OS_ERRNO_OK      = 0,
    FLEXIO_OS_ERRNO_TPT     = 10,
} flexio_os_errno_t;

struct flexio_os_thread_ctx {
    uint32_t version;
    uint16_t outbox_config_id;
    uint16_t window_config_id;
    uint64_t outbox_base;
    uint64_t window_base;
    uint64_t window_config_base;
    uint64_t metadata_parameter;
    uint32_t errno;
} __attribute__((packed, aligned(8)));

struct flexio_os_yield_ctrl {
    uint8_t valid;
    uint8_t reserved[7];
} __attribute__((packed, aligned(8)));

struct flexio_os_yield_ctx {
    struct flexio_os_yield_ctrl ctrl;
    uint8_t *ctx;
} __attribute__((packed, aligned(8)));

struct flexio_os_thread_ctx* flexio_os_get_thread_ctx(void);
struct flexio_os_thread_ctx* flexio_os_error_ctx_to_thread_ctx(uint64_t error_ctx);
uint64_t flexio_os_get_progcc_table_base(uint16_t gvmi);

typedef void (*flexio_os_yield_stateful_function_ptr)(uintptr_t arg);
//      void   flexio_os_yield_stateful_function     (uintptr_t arg); // example prototype

void flexio_os_yield_init_ctx_and_restore(uintptr_t arg, flexio_os_yield_stateful_function_ptr entry_point, uintptr_t sp, struct flexio_os_yield_ctx* yield_ctx);
void flexio_os_yield  (struct flexio_os_yield_ctx* yield_ctx);
void flexio_os_restore(struct flexio_os_yield_ctx* yield_ctx);

#endif /*__FLEXIO_OS_H__*/
